/* globals WalkMeAPI, _walkMe, WalkMeInsightsAPI */
'use strict';

var Promise = require('es6-promise').Promise;
var assign = require('lodash/assign');
var find = require('lodash/find');
var findLast = require('lodash/findLast');
var sortBy = require('lodash/sortBy');

var wrapper = {};
var API_LOAD_INTERVAL = 500;
var API_LOAD_ATTEMPTS = 120;
var isLoadedPromise = void 0;

var StepType = {
  Step: 0,
  PopupStep: 1,
  SuperStep: 2
};

wrapper.Event = {
  AfterMenuClose: 'AfterMenuClose',
  AfterMenuOpen: 'AfterMenuOpen',
  BBCodeLinkClicked: 'BBCodeLinkClicked',
  BackButtonClicked: 'BackButtonClicked',
  BalloonCreated: 'BalloonCreated',
  BeforeMenuClose: 'BeforeMenuClose',
  BeforeMenuOpen: 'BeforeMenuOpen',
  DoneButtonClicked: 'DoneButtonClicked',
  LauncherClicked: 'LauncherClicked',
  LauncherIsShown: 'LauncherIsShown',
  NextButtonClicked: 'NextButtonClicked',
  NextStepShown: 'NextStepShown',
  PlayerInitComplete: 'PlayerInitComplete',
  SurveyPlayed: 'SurveyPlayed',
  SurveyCompleted: 'SurveyCompleted',
  TaskCompleted: 'TaskCompleted',
  TasksLoaded: 'TasksLoaded',
  UserStoppedWalkthru: 'UserStoppedWalkthru',
  UserStoppedWalkthruAfterStop: 'UserStoppedWalkthruAfterStop',
  WalkthruAborted: 'WalkthruAborted',
  WalkthruFailedToStart: 'WalkthruFailedToStart',
  WalkthruSelected: 'WalkthruSelected'
};

var afterApiLoad = function afterApiLoad(fn) {
  if (!isLoadedPromise) {
    isLoadedPromise = new Promise(function isLoaded(resolve, reject) {
      var count = 1;
      var interval = setInterval(function () {
        if (typeof WalkMeAPI !== 'undefined') {
          resolve(true);
          clearInterval(interval);
        } else if (count === API_LOAD_ATTEMPTS) {
          reject(new Error(false));
          clearInterval(interval);
        }
        count++;
      }, API_LOAD_INTERVAL);
    });
  }

  return function isLoaded() {
    var args = arguments;
    return isLoadedPromise.then(function () {
      return fn.apply(null, args);
    }).catch(function () {
      return false;
    });
  };
};

wrapper.runSnippet = function runSnippet(walkmeSnippetUrl) {
  var walkme = document.createElement('script');
  var script = void 0;
  walkme.type = 'text/javascript';
  walkme.async = true;
  walkme.src = walkmeSnippetUrl;
  script = document.getElementsByTagName('script')[0];
  script.parentNode.insertBefore(walkme, script);
  window._walkmeConfig = { smartLoad: true };
};

wrapper.startWalkthrough = afterApiLoad(function (id) {
  WalkMeAPI.startWalkthruById(id);
});

wrapper.isLoaded = afterApiLoad(function () {
  return true;
});

wrapper.startVisionsRecording = function startVisionsRecording() {
  if (WalkMeInsightsAPI) {
    WalkMeInsightsAPI.startPlaybackRecording();
  } else {
    throw new Error('Visions API not loaded.');
  }
};

var eventSubscribers = Object.keys(wrapper.Event).reduce(function (subscribers, event) {
  subscribers[event] = [];
  return subscribers;
}, {});

/**
 * Listen to a WalkMe event
 *
 * @param  {String} eventName - the WalkMe event
 * @param  {Function} handler - the method to execute when this event is received
 */
wrapper.on = afterApiLoad(function (eventName, handler) {
  window.menu_event = window.menu_event || function onMenuEvent(eventData) {
    eventSubscribers[eventData.Type].forEach(function (subscriber) {
      subscriber(eventData);
    });
  };

  window.walkme_event = window.walkme_event || function onWalkMeEvent(eventData) {
    eventSubscribers[eventData.Type].forEach(function (subscriber) {
      subscriber(eventData);
    });
  };

  eventSubscribers[eventName].push(handler);
});

var lastStepMemo = {};
function isLastStep(walkthruId, stepIndex) {
  if (walkthruId in lastStepMemo) {
    return lastStepMemo[walkthruId] === stepIndex;
  }

  var isUndocumentedApiAvailable = _walkMe && typeof _walkMe.getTutorials === 'function';

  if (isUndocumentedApiAvailable) {
    var tutorial = find(_walkMe.getTutorials(), { Id: walkthruId });
    var sortedSteps = sortBy(tutorial.Steps, 'StepIndex');
    var lastStep = tutorial && findLast(sortedSteps, function (step) {
      return !step.IsSkippable && step.Status === 1 && step.Type !== StepType.SuperStep;
    });

    if (typeof lastStep !== 'undefined') {
      lastStepMemo[walkthruId] = lastStep.StepIndex;
      return lastStep.StepIndex === stepIndex;
    }
  }

  return false;
}

wrapper.onWalkthroughCompleted = function onWalkthroughCompleted(handler) {
  function onAllStepsFinished(eventData) {
    if (isLastStep(eventData.WalkthruId, eventData.StepIndex)) {
      handler(eventData);
    }
  }

  wrapper.on(wrapper.Event.TaskCompleted, handler);
  wrapper.on(wrapper.Event.NextButtonClicked, onAllStepsFinished);
  wrapper.on(wrapper.Event.DoneButtonClicked, onAllStepsFinished);
};

/**
 * addVariables attaches a "CustomWalkMeCompanyVariables" object
 * to window with the given values.
 *
 * This method can be called multiple times and the new variables
 * will be merged with the existing variables
 *
 * @param {Object} variables - a map of variable name to value
 */
wrapper.addVariables = function addVariables(variables) {
  window.CustomWalkMeCompanyVariables = assign(window.CustomWalkMeCompanyVariables, variables);
};

module.exports = wrapper;